use axum::http::header::ToStrError;
use thiserror::Error;

#[derive(Error, Debug, Copy, Clone)]
pub enum FoxError {
    #[error("object not found")]
    NotFound,
    #[error("date for signature out of range")]
    SignatureDateOutOfRange(&'static str),
    #[error("database error")]
    DatabaseError,
    #[error("non-200 response to federation request")]
    ResponseNotOk,
    #[error("server is invalid")]
    InvalidServer,
    #[error("invalid header")]
    InvalidHeader,
    #[error("invalid date format")]
    InvalidDate,
    #[error("missing signature")]
    MissingSignature,
    #[error("invalid signature")]
    InvalidSignature,
    #[error("missing or invalid token")]
    Unauthorized,
    #[error("missing target user ID")]
    MissingUser,
    #[error("user is not in guild or guild doesn't exist")]
    NotInGuild,
    #[error("channel not found")]
    ChannelNotFound,
    #[error("internal server error while proxying")]
    ProxyInternalServerError,
}

impl From<ToStrError> for FoxError {
    fn from(_: ToStrError) -> Self {
        Self::InvalidHeader
    }
}

impl From<chrono::ParseError> for FoxError {
    fn from(_: chrono::ParseError) -> Self {
        Self::InvalidDate
    }
}
