use chrono::{DateTime, Utc};
use serde::{Deserialize, Serialize};

use crate::{
    id::GuildType,
    model::{user::PartialUser, Guild, Message, User},
    Id,
};

#[derive(Debug, Serialize, Deserialize, Clone)]
#[serde(tag = "t", content = "d", rename_all = "SCREAMING_SNAKE_CASE")]
pub enum Dispatch {
    MessageCreate {
        id: String,
        channel_id: String,
        guild_id: String,
        author: PartialUser,
        content: Option<String>,
        created_at: DateTime<Utc>,
    },
    Ready {
        user: User,
        guilds: Vec<Guild>,
    }
}

impl Dispatch {
    pub fn message_create(m: Message, guild_id: Id<GuildType>) -> Self {
        Self::MessageCreate {
            id: m.id,
            channel_id: m.channel_id,
            guild_id: guild_id.0,
            author: m.author,
            content: m.content,
            created_at: m.created_at,
        }
    }
}
