use serde::{Deserialize, Serialize};

use super::Dispatch;

#[derive(Debug, Serialize, Deserialize)]
#[serde(tag = "t", content = "d", rename_all = "SCREAMING_SNAKE_CASE")]
pub enum Payload {
    #[serde(rename = "D")]
    Dispatch {
        #[serde(rename = "e")]
        event: Dispatch,
        #[serde(rename = "r")]
        recipients: Vec<String>,
    },
    Error {
        message: String,
    },
    /// Hello message, sent after authentication succeeds
    Hello {
        heartbeat_interval: u64,
    },
    /// S2S authentication. Fields correspond to headers (Host, Date, X-Foxchat-Server, X-Foxchat-Signature)
    Identify {
        host: String,
        date: String,
        server: String,
        signature: String,
    },
    /// Sent when a user connects to the identity server's gateway, to signal the chat server to send READY for that user
    Connect {
        user_id: String,
    },
    /// Sent on a regular interval by the connecting server, to keep the connection alive.
    Heartbeat {
        #[serde(rename = "t")]
        timestamp: u64,
    },
    /// Sent in response to a Heartbeat.
    HeartbeatAck {
        #[serde(rename = "t")]
        timestamp: u64,
    }
}
